--WEAPONS.LUA--
local path = mod_loader.mods[modApi.currentMod].resourcePath
local scriptpath = mod_loader.mods[modApi.currentMod].scriptPath
local getModUtils = require(path .."scripts/getModUtils")
local scriptpath = mod_loader.mods[modApi.currentMod].scriptPath
local weaponApi = require(scriptpath .."weapons/api")

local this = {}

local function IsTipImage()
	return Board:GetSize() == Point(6,6)
end

--Yeet Squad Weapon Definitions--
--Prime Throw (Prime)--
xen_Prime_Yeet = Skill:new{
	Name = "Prime Throw",
	Description = "Throw targets, creating an explosion where the target lands. Throw farther to do more damage.",
	Class = "Prime",
	Icon = "weapons/xen_weapon_prime_throw.png",
	Damage = 2,
	MinDamage = 1,
	SplashDamage = 0,
	PowerCost = 2,
	Upgrades = 2,
	AllyDamage = 0,
	BuildingDamage = true,
	UpgradeList = {"Ally & Range","Building & Range"},
	UpgradeCost = {2,2},
	Range = 3,
	LaunchSound = "/weapons/shift",
	TipImage = {
		Unit = Point(2,3),
		Target = Point(2,1),
		Enemy1 = Point(2,4),
		Enemy2 = Point(2,0),
		Building = Point(3,1),
		Building2 = Point(3,2),
		Friendly = Point(1,1),
		Friendly2 = Point(1,3),
		Second_Origin = Point(2,3),
		Second_Target = Point(3,3),
		}
}

function xen_Prime_Yeet:GetTargetArea(p1)
local ret = PointList()
  for dir = DIR_START, DIR_END do
    local curr = p1 - DIR_VECTORS[dir]
    if Board:IsPawnSpace(curr) and not Board:GetPawn(curr):IsGuarding() then
    	local throwrange = self.Range
    	for i = 1, throwrange do
    		local curr = p1 + DIR_VECTORS[dir]*i
    		if not Board:IsBlocked(curr, PATH_FLYER) then
    		ret:push_back(curr)
    		end
    	end
    end
  end
return ret
end

function xen_Prime_Yeet:GetSkillEffect(p1, p2)
	local ret = SkillEffect()
	local direction = GetDirection(p2 - p1)
	local target = p1+DIR_VECTORS[(direction+2)%4]
	ret:AddMelee(p1,SpaceDamage(target,0))
	local move = PointList()
	move:push_back(p1-DIR_VECTORS[direction])
	move:push_back(p2)
	ret:AddLeap(move, FULL_DELAY)
	
	local distance = p1:Manhattan(p2)
	--LOG(distance)

	local centre = SpaceDamage(p2,self.MinDamage + math.floor((distance / 2 )))
	if Board:IsPawnTeam(p1-DIR_VECTORS[direction],TEAM_PLAYER) then
		centre.iDamage = centre.iDamage - self.AllyDamage
		if centre.iDamage < 1 then
			centre.iDamage = 0
		end
	end
	ret:AddDamage(centre)
	ret:AddBounce(centre.loc,10)

	if distance > 1 then
		for i = -1, 1 do
			for j = -1, 1 do
				if i ~= j and i ~= -j then
					local damage = SpaceDamage(p2+Point(i,j),self.SplashDamage + math.floor((distance / 2 )))
					damage.sAnimation = "ExploArt2"
					if Board:IsBuilding(p2+Point(i,j)) and self.BuildingDamage == false then
						damage.iDamage = 0
					end
					if Board:IsPawnTeam(p2+Point(i,j),TEAM_PLAYER) then
						damage.iDamage = damage.iDamage - self.AllyDamage
						if damage.iDamage < 1 then
							damage.iDamage = 0
						end
					end
					ret:AddDamage(damage)		
				end
			end
		end
	end
	return ret
end


xen_Prime_Yeet_A = xen_Prime_Yeet:new{
	UpgradeDescription = "Reduce damage to friendly units by 1 and increase maximum throw distance by 1.",
	Range = 4,
	AllyDamage = 1,
	Damage = 3,
}

xen_Prime_Yeet_B = xen_Prime_Yeet:new{
	UpgradeDescription = "Buildings take no damage and increase maximum throw distance by 1.",
	Range = 4,
	Damage = 3,
	BuildingDamage = false,
}

xen_Prime_Yeet_AB = xen_Prime_Yeet:new{
	Range = 5,
	AllyDamage = 1,
	Damage = 3,
	BuildingDamage = false,
}

--Recall Teleport (Science)--
xen_Science_RecallTeleporter = Skill:new{
	Name = "Recall Teleporter",
	Class = "Science",
	Description = "Teleport next to an allied unit, self-repair, and pull in units.",
	Icon = "weapons/xen_weapon_science_recall.png",
	Rarity = 1,
	Explosion = "",
--	LaunchSound = "/weapons/titan_fist",
	Damage = 0,
	Repair = -2,
	Shield = false,
	Acid = false,
	PowerCost = 1,
	Upgrades = 1,
	UpgradeList = { "A.C.I.D. Explosion" },
	UpgradeCost = { 2 },
	LaunchSound = "/weapons/swap",
	Animation = "ExploRepulse1",
	TipImage = {
		Unit = Point(2,2),
		Enemy1 = Point(4,2),
		Enemy2 = Point(1,2),
		Enemy3 = Point(2,0),
		Building = Point(2,1),
		Enemy4 = Point(2,3),
		Enemy5 = Point(2,4),
		Target = Point(0,2),
		Friendly = Point(0,1)
		}	
}
function xen_Science_RecallTeleporter:GetTargetArea(point)
	local ret = PointList()
	local pawn = Board:GetPawn(point)
	local mechs = extract_table(Board:GetPawns(TEAM_MECH))
	for i, id in ipairs(mechs) do
		for dir = DIR_START, DIR_END do
			local curr = Board:GetPawn(id):GetSpace() + DIR_VECTORS[dir]
			if Board:GetPawn(id):GetSpace() ~= point and
			curr ~= point + Point(0,1) and
			curr ~= point + Point(1,0) and
			curr ~= point + Point(0,-1) and
			curr ~= point + Point(-1,0) then
				if not Board:IsBlocked(curr, PATH_FLYER) and Board:IsValid(curr) then
					ret:push_back(curr)
				end
			end
		end
	end
	if IsTestMechScenario() then
		local board_size = Board:GetSize()
		local p = {}
		for i = 0, board_size.x - 1 do
			for j = 0, board_size.y - 1 do
				if not Board:IsBlocked(Point(i,j),PATH_FLYER) then
					p[#p+1] = Point(i,j)
				end
			end
		end
		--LOG(save_table(p))
		
		for i, id in pairs(p) do
			if not Board:IsBlocked(id, PATH_FLYER) and Board:IsValid(id) then
				if id ~= point + Point(0,1) and
				id ~= point + Point(1,0) and
				id ~= point + Point(0,-1) and
				id ~= point + Point(-1,0) then
					ret:push_back(id)
				end
			end
		end
	end
	return ret
end

function xen_Science_RecallTeleporter:GetSkillEffect(p1, p2)
	local ret = SkillEffect()
	
	local animation = SpaceDamage(p1,0)
	animation.sAnimation = self.Animation
	ret:AddDamage(animation)	
	
	for dir = DIR_START, DIR_END do
		local space = p1 + DIR_VECTORS[dir]*2
		local damage = SpaceDamage(space, 0)
		local direction = GetDirection(p1 - space)
		if not Board:IsBuilding(p1 + DIR_VECTORS[dir]) then
			if not Board:IsTerrain(p1 + DIR_VECTORS[dir], TERRAIN_MOUNTAIN) then
				damage.iPush = direction
				ret:AddDamage(damage)
			end
		end
		if self.Acid then
			damage = SpaceDamage(p1 + DIR_VECTORS[dir], 0)
			damage.iAcid = EFFECT_CREATE
			ret:AddDamage(damage)
		end
	end

	local delay = Board:IsPawnSpace(p2) and 0 or FULL_DELAY
	ret:AddTeleport(p1,p2, delay)
	if delay ~= FULL_DELAY then
		ret:AddTeleport(p2,p1, FULL_DELAY)
	end
	
	if self.Repair < 0 then
		local repair = SpaceDamage(p2,self.Repair)
		repair.iAcid = EFFECT_REMOVE
		repair.iFrozen = EFFECT_REMOVE
		repair.iFire = EFFECT_REMOVE	
		ret:AddDamage(repair)
	end
	
	if self.Shield then
		local shield = SpaceDamage(p2,0)
		shield.iShield = EFFECT_CREATE
		ret:AddDamage(shield)
	end
	
	if IsTipImage() then
		ret:AddDelay(3)
	end
	return ret
end

xen_Science_RecallTeleporter_A = xen_Science_RecallTeleporter:new{
	Acid = true,
	UpgradeDescription = "Create an A.C.I.D. explosion from your origin point."	
}

--Teleport Push (Science)--
xen_Science_PushTeleporter = Skill:new{
	Name = "Push Teleporter",
	Class = "Science",
	Description = "Teleport a target one space backwards.",
	Icon = "weapons/science_pushbeam.png",
	Rarity = 1,
	Explosion = "",
	Damage = 0,
	Range = 2,
	PowerCost = 0,
	Upgrades = 0,
	LaunchSound = "/weapons/swap",
	TipImage = {
		Unit = Point(2,3),
		Enemy1 = Point(2,2),
		Enemy2 = Point(2,1),
		Target = Point(2,1),
	}
}
function xen_Science_PushTeleporter:GetTargetArea(point)
	local ret = PointList()
	for dir = DIR_START, DIR_END do
		for range = 1, self.Range do
			local curr = point + DIR_VECTORS[dir]*range
			if (Board:IsPawnSpace(curr) and not Board:GetPawn(curr):IsGuarding()) then
				if not Board:IsBlocked(curr + DIR_VECTORS[dir], PATH_FLYER) then
					ret:push_back(curr)
				end
			end
		end
	end
	
	return ret
end
function xen_Science_PushTeleporter:GetSkillEffect(p1, p2)
	local ret = SkillEffect()
	local direction = GetDirection(p2 - p1)
	--local target = GetProjectileEnd(p1,p2)
	local delay = Board:IsPawnSpace(p2) and 0 or FULL_DELAY
	ret:AddTeleport(p2,p2+DIR_VECTORS[direction], delay)
	
	if delay ~= FULL_DELAY then
		ret:AddTeleport(p2+DIR_VECTORS[direction],p2, FULL_DELAY)
	end
	
	return ret
end

--Crush Artillery (Ranged)
xen_Ranged_CrushArtillery =	Leap_Attack:new{
	Name = "Mech-Launcher",
	Class = "Ranged",
	Description = "Fire your own mech as a projectile.",
	Icon = "weapons/xen_weapon_ranged_mechlauncher.png",	
	Rarity = 1,
	MinRange = 2,
	Range = 8,
	Cost = 1,
	--BuildingDamage = false,
	Damage = 2,
	Push = false,
	Fire = false,
	PushAnimation = 1, -- 0 = airpush; 1 = explopush1; 2 = explopush2
	PowerCost = 1,
	Upgrades = 2,
	Explosion = "ExploArt3",
	UpgradeCost = {1,2},
	UpgradeList = { "Fire", "+1 Damage & Push" },
	LaunchSound = "/weapons/artillery_volley",
	ImpactSound = "/impact/generic/explosion",
	TipImage = {
		Unit = Point(2,4),
		Enemy = Point(2,1),
		Enemy2 = Point(1,1),
		Enemy3 = Point(3,1),
		Target = Point(2,2)
	}
}

function xen_Ranged_CrushArtillery:GetTargetArea(point)
	local ret = PointList()
	
	for i = DIR_START, DIR_END do
		for k = self.MinRange, self.Range do
			local curr = DIR_VECTORS[i]*k + point
			if Board:IsValid(curr) and not Board:IsBlocked(curr, Pawn:GetPathProf()) then
				ret:push_back(DIR_VECTORS[i]*k + point)
			end
		end
	end
	
	return ret
end

function xen_Ranged_CrushArtillery:GetSkillEffect(p1, p2)
	local ret = SkillEffect()
	local direction = GetDirection(p2 - p1)
	
	local move = PointList()
	move:push_back(p1)
	move:push_back(p2)
	ret:AddBounce(p1,5)
	ret:AddLeap(move, FULL_DELAY)

	ret:AddBounce(p2,5)
	local damage = SpaceDamage(p2, 0)
	damage.sAnimation = "airpush_".. (direction)
	ret:AddDamage(damage)	

	ret:AddDelay(0.1)
	
	damage = SpaceDamage(p2 + DIR_VECTORS[direction], self.Damage)
	damage.sAnimation = self.Explosion
	if self.Fire then
		damage.iFire = EFFECT_CREATE
	end
	damage.iPush = direction
	ret:AddDamage(damage)
		
	if self.Push then
		damage = SpaceDamage(p2 + DIR_VECTORS[direction] + (DIR_VECTORS[(direction+1)%4]), 0)
		damage.sAnimation = "airpush_".. (direction)
		damage.iPush = direction
		ret:AddDamage(damage)
					
		damage = SpaceDamage(p2 + DIR_VECTORS[direction] + (DIR_VECTORS[(direction-1)%4]), 0)
		damage.sAnimation = "airpush_".. (direction)
		damage.iPush = direction
		ret:AddDamage(damage)
	end
	
	ret:AddDelay(0.1)
	
	if self.Fire then
		damage = SpaceDamage(p2 + DIR_VECTORS[direction] * 2, 0)
		damage.iFire = EFFECT_CREATE
		ret:AddDamage(damage)
	end
	

	return ret
	
end

xen_Ranged_CrushArtillery_A = xen_Ranged_CrushArtillery:new{
	Fire = true,
	UpgradeDescription = "Burn the target."
}
xen_Ranged_CrushArtillery_B = xen_Ranged_CrushArtillery:new{
	Damage = 3,
	Push = true,
	UpgradeDescription = "Increase damage by 1 and push more targets."
}
xen_Ranged_CrushArtillery_AB = xen_Ranged_CrushArtillery:new{
	Damage = 3,
	Push = true,
	Fire = true
}

xen_Passive_NullZone = PassiveSkill:new{
	Name = "Dampening Field",
	Description = "Generates a field that reduces damage from all attacks started from inside it.",
	PowerCost = 1,
	Passive = "Dampening_Field",
	Icon = "weapons/xen_weapon_passive_nullzone.png",
	Upgrades = 2,
	UpgradeCost = {1,3},
	UpgradeList = { "-1 Damage", "-1 Damage"},
	Damage = -1,
	TipImage = {
		Unit = Point(2,2),
		CustomPawn = "Leaper2",
		Target = Point(2,1),
	}
}

function xen_Passive_NullZone:GetSkillEffect(p1, p2)
	--For the TipImage only
	local pawn = PAWN_FACTORY:CreatePawn("Leaper2")
	Board:AddPawn(pawn)
	pawn:SetSpace(Point(1,3))
	
	local pawn = PAWN_FACTORY:CreatePawn("xen_AssistMechTip")
	Board:AddPawn(pawn)
	pawn:SetSpace(Point(2,1))
	
	local pawn = PAWN_FACTORY:CreatePawn("xen_ThrowMechTip")
	Board:AddPawn(pawn)
	pawn:SetSpace(Point(1,2))
		
	local ret = SkillEffect()
		
	local string = "Field: " .. self.Damage
	local zone = {
		Point(2,1),
		Point(2,2),
		Point(2,0),
		Point(1,1),
		Point(3,1),
	}
	Board:AddAlert(zone[1],string)
	for i,point in pairs(zone) do
		Board:Ping(point, GL_Color(250,40,30))
	end
	
	damage = SpaceDamage(Point(2,1), 5 + self.Damage)
	damage.sAnimation = "SwipeClaw2"
	ret:AddDamage(damage)	
	
	damage = SpaceDamage(Point(1,2), 5)
	damage.sAnimation = "SwipeClaw2"
	ret:AddDamage(damage)
	
	ret:AddDelay(1)

	return ret
end

xen_Passive_NullZone_A = xen_Passive_NullZone:new{
	Damage = -2,
	UpgradeDescription = "Damage reduced by 1",
	Passive = "Dampening_Field_A"
}
xen_Passive_NullZone_B = xen_Passive_NullZone:new{
	Damage = -2,
	UpgradeDescription = "Damage reduced by 1",
	Passive = "Dampening_Field_B"
}
xen_Passive_NullZone_AB = xen_Passive_NullZone:new{
	Damage = -3,
	Passive = "Dampening_Field_AB"
}

local NullZone_strengthlist = {
	["xen_Passive_NullZone"] = -1,
	["xen_Passive_NullZone_A"] = -2,
	["xen_Passive_NullZone_B"] = -2,
	["xen_Passive_NullZone_AB"] = -3,
}

local function ClearMissionData(mission)
	if not mission then return end
	mission.xen_NullZone_Tiles = {}
	mission.xen_NullZone_Strength = 0
end

local function Create_NullZone(mission)
	if not mission then return end
	if IsTestMechScenario() then return end
	if IsTipImage() then return end
	local zone = {}
	local strength = 0
	
	local mechs = extract_table(Board:GetPawns(TEAM_MECH))
	--LOG(save_table(mechs))
	for i, id in ipairs(mechs) do
		local curr = id
		local weapons = weaponApi.GetCurrent(curr)		
		--LOG("Weapons:".. save_table(weapons))
		local s = weapons[1] or nil
		if s ~= nil then
			s = s:sub(1,20)
			--LOG(s)
			if s == "xen_Passive_NullZone" then
					--p[#p+1] = Point(i,j)
				if Board:IsValid(Board:GetPawn(curr):GetSpace()) then
					zone[#zone+1] = Board:GetPawn(curr):GetSpace()
				end
				for dir = DIR_START,DIR_END do
					if Board:IsValid(Board:GetPawn(curr):GetSpace() + DIR_VECTORS[dir]) then
						zone[#zone+1] = Board:GetPawn(curr):GetSpace() + DIR_VECTORS[dir]
					end
				end
				strength = NullZone_strengthlist[weapons[1]]
				
			end
		end
		
		s = weapons[2] or nil
		if s ~= nil then
			s = s:sub(1,20)
			--LOG(s)
			if s == "xen_Passive_NullZone" then
				if Board:IsValid(Board:GetPawn(curr):GetSpace()) then
					zone[#zone+1] = Board:GetPawn(curr):GetSpace()
				end
				for dir = DIR_START,DIR_END do
					if Board:IsValid(Board:GetPawn(curr):GetSpace() + DIR_VECTORS[dir]) then
						zone[#zone+1] = Board:GetPawn(curr):GetSpace() + DIR_VECTORS[dir]
					end
				end
				strength = NullZone_strengthlist[weapons[2]]
			end
		end		
	end
	mission.xen_NullZone_Tiles = zone
	mission.xen_NullZone_Strength = strength
	local string = "Dampening Field: " .. strength
	Board:AddAlert(zone[1],string)
	for i,point in pairs(zone) do
		Board:Ping(point, GL_Color(250,40,30))
	end	
end

local function NullZone_Effect(pawn,se,p1,p2)
	if IsTipImage() then return end
	if IsTestMechScenario() then return end
	
	local weps = {"primary","secondary"}
	local modUtils = getModUtils()
	local mission = GetCurrentMission()
	if not mission then return end
	if mission.xen_NullZone_Strength == 0 then return end --Don't run if no passive setup
	local strength = mission.xen_NullZone_Strength
	
	local zone = mission.xen_NullZone_Tiles
	--LOG("Zone:".. save_table(zone))
	local active = false
	for i, point in pairs(zone) do
		if point == p1 then
			active = true
		end
	end
	if not active then return end
	
	for _, d in ipairs(extract_table(se.effect)) do
		if d.iDamage > 0 and d.iDamage ~= DAMAGE_ZERO and d.iDamage ~= DAMAGE_DEATH then
			d.iDamage = d.iDamage + strength
			if d.iDamage < 0 then --Ensure we don't get negative damage.
				d.iDamage = 0
			end			
		end
	end
	
	for _, d in ipairs(extract_table(se.q_effect)) do
		if d.iDamage > 0 and d.iDamage ~= DAMAGE_ZERO and d.iDamage ~= DAMAGE_DEATH then
			d.iDamage = d.iDamage + strength
			if d.iDamage < 0 then --Ensure we don't get negative damage.
				d.iDamage = 0
			end			
		end
	end
end

TILE_TOOLTIPS["null_zone"] = { "Dampening Field", "Units that attack from this space will do less damage." }
Location["combat/tile_icon/xen_tile_debuff.png"] = Point(-27, 2)

local function NullZone_Mark()
	local mission = GetCurrentMission()
	if not mission then return end
	if mission.xen_NullZone_Strength == 0 then return end --Don't run if no passive setup
	local strength = mission.xen_NullZone_Strength
	local zone = mission.xen_NullZone_Tiles
	if not zone then return end
	if #zone == 0 then return end --Don't run if no tiles
	for _, mark in ipairs(zone) do
		if not Board:IsItem(mark) and not Board:IsEnvironmentDanger(mark) then
			Board:MarkSpaceImage(mark, "combat/tile_icon/xen_tile_debuff.png", GL_Color(250,40,30))
			--Board:MarkSpaceImage(mark, "combat/tile_icon/tile_airstrike.png", GL_Color(60,110,220))
			Board:MarkSpaceDesc(mark, "null_zone")
		end
	end	
end

function xen_Ping_NullZone()
	local mission = GetCurrentMission()
	if not mission then return end
	local zone = mission.xen_NullZone_Tiles
	local strength = mission.xen_NullZone_Strength
	
	local string = "Dampening Field: " .. strength
	Board:AddAlert(zone[1],string)
	for i,point in pairs(zone) do
		Board:Ping(point, GL_Color(0, 0, 255))
	end	
end

function this:init(mod)
end

function this:load(modApiExt)
	local this = self
    local modUtils = getModUtils()		
	modApi:addMissionStartHook(function(mission)
		--LOG("Yeetsquad: Start mission hook")
		ClearMissionData(mission)
	end)
	modApi:addNextTurnHook(function(mission)
		--LOG("Yeetsquad: Next turn hook")
		if Game:GetTeamTurn() == TEAM_PLAYER then
			Create_NullZone(mission)
		end
	end)
	modUtils:addSkillBuildHook(function(mission,pawn,weaponId,p1,p2,skillEffect)
		--LOG("Yeetsquad: Skill Build Hook")
		NullZone_Effect(pawn,skillEffect,p1,p2)
	end)
	modApi:addMissionUpdateHook(function(mission)
		NullZone_Mark()
	end)
end

return this
